@@ -1,8 +1,7 @@
-# from __future__ import absolute_import, division, generate_function
-# __metaclass__ = type
 import json
 import requests
 import re
+import sys
 import time
 from ansible.module_utils.basic import AnsibleModule, json, env_fallback
 from collections import OrderedDict
@@ -12,7 +11,6 @@ try:
 except ImportError:
     JSONDecodeError = ValueError
 
-
 def viptela_argument_spec():
     return dict(host=dict(type='str', required=True, fallback=(env_fallback, ['VMANAGE_HOST'])),
                 port=dict(type='str', required=False, fallback=(env_fallback, ['VMANAGE_PORT'])),
@@ -77,7 +75,7 @@ class viptelaModule(object):
         return value
 
     def list_to_dict(self, list, key_name, remove_key=True):
-        dict = OrderedDict()
+        dict_value = OrderedDict()
         for item in list:
             if key_name in item:
                 if remove_key:
@@ -85,11 +83,11 @@ class viptelaModule(object):
                 else:
                     key = item[key_name]
 
-                dict[key] = item
+                dict_value[key] = item
             # else:
             #     self.fail_json(msg="key {0} not found in dictionary".format(key_name))
 
-        return dict
+        return dict_value
 
     @staticmethod
     def compare_payloads(new_payload, old_payload, compare_values=None):
@@ -98,9 +96,7 @@ class viptelaModule(object):
         payload_key_diff = []
         for key, value in new_payload.items():
             if key in compare_values:
-                if key not in old_payload:
-                    payload_key_diff.append(key)
-                elif new_payload[key] != old_payload[key]:
+                if key not in old_payload or new_payload[key] != old_payload[key]:
                     payload_key_diff.append(key)
         return payload_key_diff
 
@@ -150,9 +146,6 @@ class viptelaModule(object):
         if files is None:
             self.session.headers['Content-Type'] = 'application/json'
 
-        # {'Connection': 'keep-alive', 'Content-Type': 'application/json'}
-        # self.session.headers.update(headers)
-
         if payload:
             self.result['payload'] = payload
             data = json.dumps(payload)
@@ -259,13 +252,13 @@ class viptelaModule(object):
             for device in device_body['data']:
                 object_response = self.request('/dataservice/template/device/object/{0}'.format(device['templateId']))
                 if object_response.json:
-                    object = object_response.json
-                    if not factory_default and object['factoryDefault']:
+                    object_value = object_response.json
+                    if not factory_default and object_value['factoryDefault']:
                         continue
 
-                    if 'generalTemplates' in object:
+                    if 'generalTemplates' in object_value:
                         generalTemplates = []
-                        for old_template in object.pop('generalTemplates'):
+                        for old_template in object_value.pop('generalTemplates'):
                             new_template = {
                                 'templateName': feature_template_dict[old_template['templateId']]['templateName'],
                                 'templateType': old_template['templateType']}
@@ -277,13 +270,13 @@ class viptelaModule(object):
                                 new_template['subTemplates'] = subTemplates
 
                             generalTemplates.append(new_template)
-                        object['generalTemplates'] = generalTemplates
+                        object_value['generalTemplates'] = generalTemplates
 
-                    object['templateId'] = device['templateId']
-                    object['attached_devices'] = self.get_template_attachments(device['templateId'])
-                    object['input'] = self.get_template_input(device['templateId'])
+                    object_value['templateId'] = device['templateId']
+                    object_value['attached_devices'] = self.get_template_attachments(device['templateId'])
+                    object_value['input'] = self.get_template_input(device['templateId'])
 
-                    return_list.append(object)
+                    return_list.append(object_value)
 
         return return_list
 
@@ -357,8 +350,6 @@ class viptelaModule(object):
 
     def get_vmanage_vbond(self):
         response = self.request('/dataservice/settings/configuration/device')
-        vbond = None
-        vbond_port = None
         try:
             return {'vbond': response.json['data'][0]['domainIp'], 'vbond_port': response.json['data'][0]['port']}
         except:
@@ -366,8 +357,7 @@ class viptelaModule(object):
 
     def set_vmanage_vbond(self, vbond, vbond_port='12346'):
         payload = {'domainIp': vbond, 'port': vbond_port}
-        response = self.request('/dataservice/settings/configuration/device', method='POST', payload=payload)
-
+        self.request('/dataservice/settings/configuration/device', method='POST', payload=payload)
         return
 
     def get_vmanage_ca_type(self):
@@ -379,7 +369,7 @@ class viptelaModule(object):
 
     def set_vmanage_ca_type(self, type):
         payload = {'certificateSigning': type, 'challengeAvailable': 'false'}
-        response = self.request('/dataservice/settings/configuration/certificate', method='POST', payload=payload)
+        self.request('/dataservice/settings/configuration/certificate', method='POST', payload=payload)
         return
 
     def get_vmanage_root_cert(self):
@@ -391,7 +381,7 @@ class viptelaModule(object):
 
     def set_vmanage_root_cert(self, cert):
         payload = {'enterpriseRootCA': cert}
-        response = self.request('/dataservice/settings/configuration/certificate/enterpriserootca', method='PUT', payload=payload)
+        self.request('/dataservice/settings/configuration/certificate/enterpriserootca', method='PUT', payload=payload)
         return
 
     def install_device_cert(self, cert):
@@ -517,14 +507,6 @@ class viptelaModule(object):
 
         return self.list_to_dict(device_list, key_name=key_name, remove_key=remove_key)
 
-    def get_device_status(self, value, key='system-ip'):
-        response = self.request('/dataservice/device?{0}={1}'.format(key, value))
-
-        try:
-            return response.json['data'][0]
-        except:
-            return {}
-
     def get_device_vedges(self, key_name='host-name', remove_key=True):
         response = self.request('/dataservice/system/device/vedges')
 
