@@ -59,8 +59,10 @@ class PolicyData(object):
                                                                       cache=False)
             if policy_list['name'] in policy_list_dict:
                 existing_list = policy_list_dict[policy_list['name']]
-                diff_ignore = set(
-                    ['listId', 'references', 'lastUpdated', 'activatedId', 'policyId', 'listId', 'isActivatedByVsmart'])
+                diff_ignore = set([
+                    'listId', 'referenceCount', 'references', 'owner', 'lastUpdated', 'activatedId', 'policyId',
+                    'isActivatedByVsmart'
+                ])
                 diff = list(dictdiffer.diff(existing_list, policy_list, ignore=diff_ignore))
                 if diff:
                     policy_list_updates.append({'name': policy_list['name'], 'diff': diff})
@@ -75,7 +77,7 @@ class PolicyData(object):
                             # Updating the policy list returns a `processId` that locks the list and 'masterTemplatesAffected'
                             # that lists the templates affected by the change.
                             if 'error' in response['json']:
-                                raise Exception(response['json']['error']['message'])
+                                raise RuntimeError(response['json']['error']['message'])
                             elif 'processId' in response['json']:
                                 if push:
                                     vmanage_device_templates = DeviceTemplates(self.session, self.host)
@@ -85,7 +87,7 @@ class PolicyData(object):
                                         vmanage_device_templates.reattach_device_template(
                                             template_id, obj['configType'])
                             else:
-                                raise Exception("Did not get a process id when updating policy list")
+                                raise RuntimeError("Did not get a process id when updating policy list")
             else:
                 diff = list(dictdiffer.diff({}, policy_list))
                 policy_list_updates.append({'name': policy_list['name'], 'diff': diff})
@@ -103,13 +105,14 @@ class PolicyData(object):
         """
         if isinstance(name_list, dict):
             for key, value in list(name_list.items()):
-                if key.endswith('List') and key != "signatureWhiteList":
+                if key.endswith(
+                        'List') and key != "signatureWhiteList" and key != "urlWhiteList" and key != "urlBlackList":
                     t = key[0:len(key) - 4]
                     policy_list = self.policy_lists.get_policy_list_by_name(value, policy_list_type=t)
                     if policy_list:
                         name_list[key] = policy_list['listId']
                     else:
-                        raise Exception(f"Could not find id for list {value}, type {t}")
+                        raise RuntimeError(f"Could not find id for list {value}, type {t}")
                 elif key.endswith('Lists'):
                     t = key[0:len(key) - 5]
                     new_list = []
@@ -119,39 +122,41 @@ class PolicyData(object):
                             list_id = policy_list['listId']
                             new_list.append(list_id)
                         else:
-                            raise Exception(f"Could not find id for list {list_name}, type {t}")
+                            raise RuntimeError(f"Could not find id for list {list_name}, type {t}")
                     name_list[key] = new_list
                 elif key.endswith('Zone'):
+                    if value == 'Self Zone':
+                        name_list[key] = 'self'
                     policy_list = self.policy_lists.get_policy_list_by_name(value, 'zone')
                     if policy_list:
                         name_list[key] = policy_list['listId']
                     else:
-                        raise Exception(f"Could not find id for list {value}, type zone")
+                        raise RuntimeError(f"Could not find id for list {value}, type zone")
                 elif key == 'listName':
                     if 'listType' in name_list:
                         policy_list = self.policy_lists.get_policy_list_by_name(name_list['listName'],
                                                                                 policy_list_type=name_list['listType'])
                     else:
-                        raise Exception(f"Could not find type for list {name_list['listName']}")
+                        raise RuntimeError(f"Could not find type for list {name_list['listName']}")
                     if policy_list and 'listId' in policy_list:
                         name_list['ref'] = policy_list['listId']
                         name_list.pop('listName')
                         name_list.pop('listType')
                     else:
-                        raise Exception(
+                        raise RuntimeError(
                             f"Could not find id for list {name_list['listName']}, type {name_list['listType']}")
                 elif key == 'className':
                     if 'classType' in name_list:
                         policy_list = self.policy_lists.get_policy_list_by_name(name_list['className'],
                                                                                 policy_list_type=name_list['classType'])
                     else:
-                        raise Exception(f"Could not find type for list {name_list['className']}")
+                        raise RuntimeError(f"Could not find type for list {name_list['className']}")
                     if policy_list and 'listId' in policy_list:
                         name_list['class'] = policy_list['listId']
                         name_list.pop('className')
                         name_list.pop('classType')
                     else:
-                        raise Exception(
+                        raise RuntimeError(
                             f"Could not find id for list {name_list['className']}, type {name_list['classType']}")
                 else:
                     self.convert_list_name_to_id(value)
@@ -168,7 +173,8 @@ class PolicyData(object):
         """
         if isinstance(id_list, dict):
             for key, value in list(id_list.items()):
-                if key.endswith('List') and key != "signatureWhiteList":
+                if key.endswith(
+                        'List') and key != "signatureWhiteList" and key != "urlWhiteList" and key != "urlBlackList":
                     t = key[0:len(key) - 4]
                     val = value
                     if isinstance(value, list):
@@ -177,7 +183,7 @@ class PolicyData(object):
                     if policy_list:
                         id_list[key] = policy_list['name']
                     else:
-                        raise Exception(f"Could not find name for list id {val}, type {t}")
+                        raise RuntimeError(f"Could not find name for list id {val}, type {t}")
                 elif key.endswith('Lists'):
                     t = key[0:len(key) - 5]
                     new_list = []
@@ -187,14 +193,17 @@ class PolicyData(object):
                             list_name = policy_list['name']
                             new_list.append(list_name)
                         else:
-                            raise Exception(f"Could not find name for list id {list_id}, type {t}")
+                            raise RuntimeError(f"Could not find name for list id {list_id}, type {t}")
                     id_list[key] = new_list
                 elif key.endswith('Zone'):
-                    policy_list = self.policy_lists.get_policy_list_by_id(value, 'zone')
-                    if policy_list:
-                        id_list[key] = policy_list['name']
+                    if value == 'self':
+                        id_list[key] = 'Self Zone'
                     else:
-                        raise Exception(f"Could not find name for list {value}, type zone")
+                        policy_list = self.policy_lists.get_policy_list_by_id(value, 'zone')
+                        if policy_list:
+                            id_list[key] = policy_list['name']
+                        else:
+                            raise RuntimeError(f"Could not find name for list {value}, type zone")
                 elif key == 'ref':
                     policy_list = self.policy_lists.get_policy_list_by_id(id_list['ref'])
                     if policy_list:
@@ -202,7 +211,7 @@ class PolicyData(object):
                         id_list['listType'] = policy_list['type']
                         id_list.pop('ref')
                     else:
-                        raise Exception(f"Could not find name for list {id_list['ref']}")
+                        raise RuntimeError(f"Could not find name for list {id_list['ref']}")
                 elif key == 'class':
                     policy_list = self.policy_lists.get_policy_list_by_id(id_list['class'])
                     if policy_list:
@@ -210,7 +219,7 @@ class PolicyData(object):
                         id_list['classType'] = policy_list['type']
                         id_list.pop('class')
                     else:
-                        raise Exception(f"Could not find name for list {id_list['class']}")
+                        raise RuntimeError(f"Could not find name for list {id_list['class']}")
                 else:
                     self.convert_list_id_to_name(value)
         elif isinstance(id_list, list):
@@ -234,7 +243,7 @@ class PolicyData(object):
                             entry.pop('listName')
                             entry.pop('listType')
                         else:
-                            raise Exception("Could not find list {0} of type {1}".format(
+                            raise RuntimeError("Could not find list {0} of type {1}".format(
                                 entry['listName'], entry['listType']))
 
     def convert_definition_id_to_name(self, policy_definition):
@@ -252,7 +261,7 @@ class PolicyData(object):
                 if policy_definition_detail:
                     assembly_item['definitionName'] = policy_definition_detail['name']
                 else:
-                    raise Exception("Cannot find policy definition for {0}".format(definition_id))
+                    raise RuntimeError("Cannot find policy definition for {0}".format(definition_id))
                 if 'entries' in assembly_item:
                     # Translate list IDs to names
                     self.convert_list_id_to_name(assembly_item['entries'])
@@ -266,12 +275,13 @@ class PolicyData(object):
         """
         if 'assembly' in policy_definition and policy_definition['assembly']:
             for assembly_item in policy_definition['assembly']:
-                definition_name = assembly_item.pop('definitionName')
+                if assembly_item['definitionName']:
+                    definition_name = assembly_item.pop('definitionName')
                 policy_definition_dict = self.policy_definitions.get_policy_definition_dict(assembly_item['type'])
                 if definition_name in policy_definition_dict:
                     assembly_item['definitionId'] = policy_definition_dict[definition_name]['definitionId']
                 else:
-                    raise Exception("Cannot find policy definition {0}".format(definition_name))
+                    raise RuntimeError("Cannot find policy definition {0}".format(definition_name))
                 if 'entries' in assembly_item:
                     self.convert_list_name_to_id(assembly_item['entries'])
 
@@ -352,6 +362,7 @@ class PolicyData(object):
 
         """
         policy_definition_updates = []
+        #pylint: disable=too-many-nested-blocks
         for definition in policy_definition_list:
             policy_definition_dict = self.policy_definitions.get_policy_definition_dict(definition['type'],
                                                                                         remove_key=False)
@@ -380,9 +391,24 @@ class PolicyData(object):
                     converted_definition = self.convert_policy_definition_to_id(definition)
                     policy_definition_updates.append({'name': converted_definition['name'], 'diff': diff})
                     if not check_mode and update:
-                        self.policy_definitions.update_policy_definition(
+                        response = self.policy_definitions.update_policy_definition(
                             converted_definition, policy_definition_dict[converted_definition['name']]['definitionId'])
-                    policy_definition_updates.append({'name': converted_definition['name'], 'diff': diff})
+
+                        if response['json']:
+                            # Updating the policy defin returns a `processId` that locks the list and 'masterTemplatesAffected'
+                            # that lists the templates affected by the change.
+                            if 'error' in response['json']:
+                                raise RuntimeError(response['json']['error']['message'])
+                            elif 'processId' in response['json']:
+                                if push:
+                                    vmanage_device_templates = DeviceTemplates(self.session, self.host)
+                                    # If told to push out the change, we need to reattach each template affected by the change
+                                    for template_id in response['json']['masterTemplatesAffected']:
+                                        obj = vmanage_device_templates.get_device_template_object(template_id)
+                                        vmanage_device_templates.reattach_device_template(
+                                            template_id, obj['configType'])
+                            else:
+                                raise RuntimeError("Did not get a process id when updating policy definition")
             else:
                 # Policy definition does not exist
                 diff = list(dictdiffer.diff({}, payload))
@@ -453,6 +479,7 @@ class PolicyData(object):
         local_policy_dict = self.local_policy.get_local_policy_dict(remove_key=False)
         diff = []
         local_policy_updates = []
+        #pylint: disable=too-many-nested-blocks
         for local_policy in local_policy_list:
             payload = {'policyName': local_policy['policyName']}
             payload['policyDescription'] = local_policy['policyDescription']
@@ -468,12 +495,25 @@ class PolicyData(object):
                 ])
                 diff = list(dictdiffer.diff(existing_policy, payload, ignore=diff_ignore))
                 if diff:
-                    print(diff)
                     local_policy_updates.append({'name': local_policy['policyName'], 'diff': diff})
                     if 'policyDefinition' in payload:
                         self.convert_definition_name_to_id(payload['policyDefinition'])
                     if not check_mode and update:
-                        self.local_policy.update_local_policy(payload, existing_policy['policyId'])
+                        response = self.local_policy.update_local_policy(payload, existing_policy['policyId'])
+
+                        if response['json']:
+                            # Updating the policy defin returns a `processId` that locks the list and 'masterTemplatesAffected'
+                            # that lists the templates affected by the change.
+                            if 'error' in response['json']:
+                                raise RuntimeError(response['json']['error']['message'])
+                            elif 'processId' in response['json']:
+                                if push:
+                                    vmanage_device_templates = DeviceTemplates(self.session, self.host)
+                                    # If told to push out the change, we need to reattach each template affected by the change
+                                    vmanage_device_templates.reattach_multi_device_templates(
+                                        response['json']['masterTemplatesAffected'])
+                            else:
+                                raise RuntimeError("Did not get a process id when updating local policy")
             else:
                 diff = list(dictdiffer.diff({}, payload['policyDefinition']))
                 local_policy_updates.append({'name': local_policy['policyName'], 'diff': diff})
@@ -513,6 +553,7 @@ class PolicyData(object):
         central_policy_dict = self.central_policy.get_central_policy_dict(remove_key=False)
         diff = []
         central_policy_updates = []
+        #pylint: disable=too-many-nested-blocks
         for central_policy in central_policy_list:
             payload = {'policyName': central_policy['policyName']}
             payload['policyDescription'] = central_policy['policyDescription']
@@ -531,7 +572,19 @@ class PolicyData(object):
                     # Convert list and definition names to template IDs
                     converted_payload = self.convert_policy_to_id(payload)
                     if not check_mode and update:
-                        self.central_policy.update_central_policy(converted_payload, existing_policy['policyId'])
+                        response = self.central_policy.update_central_policy(converted_payload,
+                                                                             existing_policy['policyId'])
+
+                        if response['json']:
+                            # Updating the policy list returns a `processId` that locks the list and 'masterTemplatesAffected'
+                            # that lists the templates affected by the change.
+                            if 'error' in response['json']:
+                                raise RuntimeError(response['json']['error']['message'])
+                            elif 'deviceId' in response['json'][0]:
+                                if push:
+                                    self.central_policy.reactivate_central_policy(existing_policy['policyId'])
+                            else:
+                                raise RuntimeError("Did not get a deviceid when updating central policy")
             else:
                 diff = list(dictdiffer.diff({}, payload['policyDefinition']))
                 central_policy_updates.append({'name': central_policy['policyName'], 'diff': diff})
